@@ -4,7 +4,7 @@
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-# http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 # EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
@@ -19,7 +19,7 @@
 # trademarks of QuantumBlack. The License does not grant you any right or
 # license to the QuantumBlack Trademarks. You may not use the QuantumBlack
 # Trademarks or any confusingly similar mark as a trademark for your product,
-#     or use the QuantumBlack Trademarks in any other manner that might cause
+# or use the QuantumBlack Trademarks in any other manner that might cause
 # confusion in the marketplace, including but not limited to in advertising,
 # on websites, or on software.
 #
@@ -80,7 +80,6 @@ class ParquetDataSet(AbstractVersionedDataSet):
         version: Version = None,
         credentials: Dict[str, Any] = None,
         fs_args: Dict[str, Any] = None,
-        layer: str = None,
     ) -> None:
         """Creates a new instance of ``ParquetDataSet`` pointing to a concrete Parquet file
         on a specific filesystem.
@@ -89,10 +88,14 @@ class ParquetDataSet(AbstractVersionedDataSet):
             filepath: Filepath to a Parquet file prefixed with a protocol like `s3://`.
                 If prefix is not provided, `file` protocol (local filesystem) will be used.
                 The prefix should be any protocol supported by ``fsspec``.
+                It can also be a path to a directory. If the directory is
+                provided then it can be used for reading partitioned parquet files.
                 Note: `http(s)` doesn't support versioning.
-            load_args: Pandas options for loading Parquet files.
-                Here you can find all available arguments:
-                https://pandas.pydata.org/pandas-docs/stable/generated/pandas.read_parquet.html
+            load_args: Additional options for loading Parquet file(s).
+                Here you can find all available arguments when reading single file:
+                https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.read_parquet.html
+                Here you can find all available arguments when reading partitioned datasets:
+                https://arrow.apache.org/docs/python/generated/pyarrow.parquet.ParquetDataset.html#pyarrow.parquet.ParquetDataset.read
                 All defaults are preserved.
             save_args: Additional saving options for `pyarrow.parquet.write_table`.
                 Here you can find all available arguments:
@@ -103,12 +106,16 @@ class ParquetDataSet(AbstractVersionedDataSet):
                 attribute is None, save version will be autogenerated.
             credentials: Credentials required to get access to the underlying filesystem.
                 E.g. for ``GCSFileSystem`` it should look like `{"token": None}`.
-            fs_args: Extra arguments to pass into underlying filesystem class.
-                E.g. for ``GCSFileSystem`` class: `{"project": "my-project", ...}`.
-            layer: The data layer according to the data engineering convention:
-                https://kedro.readthedocs.io/en/stable/06_resources/01_faq.html#what-is-data-engineering-convention
+            fs_args: Extra arguments to pass into underlying filesystem class constructor
+                (e.g. `{"project": "my-project"}` for ``GCSFileSystem``), as well as
+                to pass to the filesystem's `open` method through nested keys
+                `open_args_load` and `open_args_save`.
+                Here you can find all available arguments for `open`:
+                https://filesystem-spec.readthedocs.io/en/latest/api.html#fsspec.spec.AbstractFileSystem.open
+                All defaults are preserved.
         """
         _fs_args = deepcopy(fs_args) or {}
+        self._fs_open_args_load = _fs_args.pop("open_args_load", {})
         _credentials = deepcopy(credentials) or {}
 
         protocol, path = get_protocol_and_path(filepath, version)
@@ -123,8 +130,6 @@ class ParquetDataSet(AbstractVersionedDataSet):
             glob_function=self._fs.glob,
         )
 
-        self._layer = layer
-
         # Handle default load and save arguments
         self._load_args = deepcopy(self.DEFAULT_LOAD_ARGS)
         if load_args is not None:
@@ -140,24 +145,41 @@ class ParquetDataSet(AbstractVersionedDataSet):
             load_args=self._load_args,
             save_args=self._save_args,
             version=self._version,
-            layer=self._layer,
         )
 
     def _load(self) -> pd.DataFrame:
         load_path = get_filepath_str(self._get_load_path(), self._protocol)
 
-        with self._fs.open(load_path, mode="rb") as fs_file:
-            return pd.read_parquet(fs_file, **self._load_args)
+        if self._fs.isdir(load_path):
+            # It doesn't work at least on S3 if root folder was created manually
+            # https://issues.apache.org/jira/browse/ARROW-7867
+            data = (
+                pq.ParquetDataset(load_path, filesystem=self._fs)
+                .read(**self._load_args)
+                .to_pandas()
+            )
+        else:
+            with self._fs.open(load_path, **self._fs_open_args_load) as fs_file:
+                data = pd.read_parquet(fs_file, **self._load_args)
+
+        return data
 
     def _save(self, data: pd.DataFrame) -> None:
         save_path = get_filepath_str(self._get_save_path(), self._protocol)
 
-        table = pa.Table.from_pandas(data)
-        pq.write_table(
-            table=table, where=save_path, filesystem=self._fs, **self._save_args
-        )
+        try:
+            table = pa.Table.from_pandas(data)
+            pq.write_table(
+                table=table, where=save_path, filesystem=self._fs, **self._save_args
+            )
+        except IsADirectoryError as err:
+            raise DataSetError(
+                "Saving {} to a directory is not supported. \n{}".format(
+                    self.__class__.__name__, str(err)
+                )
+            )
 
-        self.invalidate_cache()
+        self._invalidate_cache()
 
     def _exists(self) -> bool:
         try:
@@ -168,9 +190,10 @@ class ParquetDataSet(AbstractVersionedDataSet):
         return self._fs.exists(load_path)
 
     def _release(self) -> None:
-        self.invalidate_cache()
+        super()._release()
+        self._invalidate_cache()
 
-    def invalidate_cache(self) -> None:
+    def _invalidate_cache(self) -> None:
         """Invalidate underlying filesystem caches."""
         filepath = get_filepath_str(self._filepath, self._protocol)
         self._fs.invalidate_cache(filepath)
