@@ -4,7 +4,7 @@
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-# http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 # EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
@@ -19,7 +19,7 @@
 # trademarks of QuantumBlack. The License does not grant you any right or
 # license to the QuantumBlack Trademarks. You may not use the QuantumBlack
 # Trademarks or any confusingly similar mark as a trademark for your product,
-#     or use the QuantumBlack Trademarks in any other manner that might cause
+# or use the QuantumBlack Trademarks in any other manner that might cause
 # confusion in the marketplace, including but not limited to in advertising,
 # on websites, or on software.
 #
@@ -28,8 +28,8 @@
 
 """``PickleDataSet`` loads/saves data from/to a Pickle file using an underlying
 filesystem (e.g.: local, S3, GCS). The underlying functionality is supported by
-the ``pickle`` library, so it supports all allowed options for loading and saving
-pickle files.
+the ``pickle`` and ``joblib`` libraries, so it supports all allowed options for
+loading and saving pickle files.
 """
 import pickle
 from copy import deepcopy
@@ -46,12 +46,17 @@ from kedro.io.core import (
     get_protocol_and_path,
 )
 
+try:
+    import joblib
+except ImportError:  # pragma: no cover
+    joblib = None
+
 
 class PickleDataSet(AbstractVersionedDataSet):
     """``PickleDataSet`` loads/saves data from/to a Pickle file using an underlying
     filesystem (e.g.: local, S3, GCS). The underlying functionality is supported by
-    the ``pickle`` library, so it supports all allowed options for loading and saving
-    pickle files.
+    the ``pickle`` and ``joblib`` libraries, so it supports all allowed options for
+    loading and saving pickle files.
 
     Example:
     ::
@@ -63,7 +68,7 @@ class PickleDataSet(AbstractVersionedDataSet):
         >>>                      'col3': [5, 6]})
         >>>
         >>> # data_set = PickleDataSet(filepath="gcs://bucket/test.pkl")
-        >>> data_set = PickleDataSet(filepath="test.pkl")
+        >>> data_set = PickleDataSet(filepath="test.pkl", backend="pickle")
         >>> data_set.save(data)
         >>> reloaded = data_set.load()
         >>> assert data.equals(reloaded)
@@ -72,37 +77,38 @@ class PickleDataSet(AbstractVersionedDataSet):
 
     DEFAULT_LOAD_ARGS = {}  # type: Dict[str, Any]
     DEFAULT_SAVE_ARGS = {}  # type: Dict[str, Any]
+    BACKENDS = {"pickle": pickle, "joblib": joblib}
 
     # pylint: disable=too-many-arguments
     def __init__(
         self,
         filepath: str,
+        backend: str = "pickle",
         load_args: Dict[str, Any] = None,
         save_args: Dict[str, Any] = None,
         version: Version = None,
         credentials: Dict[str, Any] = None,
         fs_args: Dict[str, Any] = None,
-        layer: str = None,
     ) -> None:
         """Creates a new instance of ``PickleDataSet`` pointing to a concrete Pickle
-        file on a specific filesystem. ``PickleDataSet`` uses `pickle` library to
-        serialize/deserialize objects.
-
-        pickle.dumps: https://docs.python.org/3/library/pickle.html#pickle.dumps
-        pickle.loads: https://docs.python.org/3/library/pickle.html#pickle.loads
+        file on a specific filesystem. ``PickleDataSet`` supports two backends to
+        serialize/deserialize objects: `pickle` and `joblib`.
 
         Args:
             filepath: Filepath to a Pickle file prefixed with a protocol like `s3://`.
                 If prefix is not provided, `file` protocol (local filesystem) will be used.
                 The prefix should be any protocol supported by ``fsspec``.
                 Note: `http(s)` doesn't support versioning.
+            backend: Backend to use, must be one of ['pickle', 'joblib']. Defaults to 'pickle'.
             load_args: Pickle options for loading pickle files.
-                Here you can find all available arguments:
-                https://docs.python.org/3/library/pickle.html#pickle.loads
+                Here you can find all available arguments for different backends:
+                pickle.load: https://docs.python.org/3/library/pickle.html#pickle.load
+                joblib.load: https://joblib.readthedocs.io/en/latest/generated/joblib.load.html
                 All defaults are preserved.
             save_args: Pickle options for saving pickle files.
-                Here you can find all available arguments:
-                https://docs.python.org/3/library/pickle.html#pickle.dumps
+                Here you can find all available arguments for different backends:
+                pickle.dump: https://docs.python.org/3/library/pickle.html#pickle.dump
+                joblib.dump: https://joblib.readthedocs.io/en/latest/generated/joblib.dump.html
                 All defaults are preserved.
             version: If specified, should be an instance of
                 ``kedro.io.core.Version``. If its ``load`` attribute is
@@ -110,12 +116,34 @@ class PickleDataSet(AbstractVersionedDataSet):
                 attribute is None, save version will be autogenerated.
             credentials: Credentials required to get access to the underlying filesystem.
                 E.g. for ``GCSFileSystem`` it should look like `{"token": None}`.
-            fs_args: Extra arguments to pass into underlying filesystem class.
-                E.g. for ``GCSFileSystem`` class: `{"project": "my-project", ...}`.
-            layer: The data layer according to the data engineering convention:
-                https://kedro.readthedocs.io/en/stable/06_resources/01_faq.html#what-is-data-engineering-convention
+            fs_args: Extra arguments to pass into underlying filesystem class constructor
+                (e.g. `{"project": "my-project"}` for ``GCSFileSystem``), as well as
+                to pass to the filesystem's `open` method through nested keys
+                `open_args_load` and `open_args_save`.
+                Here you can find all available arguments for `open`:
+                https://filesystem-spec.readthedocs.io/en/latest/api.html#fsspec.spec.AbstractFileSystem.open
+                All defaults are preserved, except `mode`, which is set to `wb` when saving.
+
+        Raises:
+            ValueError: If ``backend`` is not one of ['pickle', 'joblib'].
+            ImportError: If ``backend`` library could not be imported.
         """
+        if backend not in self.BACKENDS:
+            raise ValueError(
+                "'backend' should be one of {}, got '{}'.".format(
+                    list(self.BACKENDS.keys()), backend
+                )
+            )
+
+        if not self.BACKENDS[backend]:
+            raise ImportError(
+                "Selected backend '{}' could not be "
+                "imported. Make sure it is installed.".format(backend)
+            )
+
         _fs_args = deepcopy(fs_args) or {}
+        _fs_open_args_load = _fs_args.pop("open_args_load", {})
+        _fs_open_args_save = _fs_args.pop("open_args_save", {})
         _credentials = deepcopy(credentials) or {}
 
         protocol, path = get_protocol_and_path(filepath, version)
@@ -130,7 +158,7 @@ class PickleDataSet(AbstractVersionedDataSet):
             glob_function=self._fs.glob,
         )
 
-        self._layer = layer
+        self._backend = backend
 
         # Handle default load and save arguments
         self._load_args = deepcopy(self.DEFAULT_LOAD_ARGS)
@@ -140,39 +168,42 @@ class PickleDataSet(AbstractVersionedDataSet):
         if save_args is not None:
             self._save_args.update(save_args)
 
+        _fs_open_args_save.setdefault("mode", "wb")
+        self._fs_open_args_load = _fs_open_args_load
+        self._fs_open_args_save = _fs_open_args_save
+
     def _describe(self) -> Dict[str, Any]:
         return dict(
             filepath=self._filepath,
+            backend=self._backend,
             protocol=self._protocol,
             load_args=self._load_args,
             save_args=self._save_args,
             version=self._version,
-            layer=self._layer,
         )
 
     def _load(self) -> Any:
         load_path = get_filepath_str(self._get_load_path(), self._protocol)
 
-        with self._fs.open(load_path, mode="rb") as fs_file:
-            return pickle.loads(fs_file.read(), **self._load_args)
+        with self._fs.open(load_path, **self._fs_open_args_load) as fs_file:
+            return self.BACKENDS[self._backend].load(
+                fs_file, **self._load_args
+            )  # nosec
 
     def _save(self, data: Any) -> None:
         save_path = get_filepath_str(self._get_save_path(), self._protocol)
 
-        try:
-            bytes_object = pickle.dumps(data, **self._save_args)
-        except pickle.PickleError:
-            raise DataSetError(
-                "{} cannot be serialized. {} can only be used with "
-                "serializable data".format(
-                    str(data.__class__), str(self.__class__.__name__)
+        with self._fs.open(save_path, **self._fs_open_args_save) as fs_file:
+            try:
+                self.BACKENDS[self._backend].dump(data, fs_file, **self._save_args)
+            except Exception as err:
+                raise DataSetError(
+                    "{} was not serialized due to: {}".format(
+                        str(data.__class__), str(err)
+                    )
                 )
-            )
 
-        with self._fs.open(save_path, mode="wb") as fs_file:
-            fs_file.write(bytes_object)
-
-        self.invalidate_cache()
+        self._invalidate_cache()
 
     def _exists(self) -> bool:
         try:
@@ -183,9 +214,10 @@ class PickleDataSet(AbstractVersionedDataSet):
         return self._fs.exists(load_path)
 
     def _release(self) -> None:
-        self.invalidate_cache()
+        super()._release()
+        self._invalidate_cache()
 
-    def invalidate_cache(self) -> None:
+    def _invalidate_cache(self) -> None:
         """Invalidate underlying filesystem caches."""
         filepath = get_filepath_str(self._filepath, self._protocol)
         self._fs.invalidate_cache(filepath)
