@@ -4,7 +4,7 @@
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-# http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 # EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
@@ -19,7 +19,7 @@
 # trademarks of QuantumBlack. The License does not grant you any right or
 # license to the QuantumBlack Trademarks. You may not use the QuantumBlack
 # Trademarks or any confusingly similar mark as a trademark for your product,
-#     or use the QuantumBlack Trademarks in any other manner that might cause
+# or use the QuantumBlack Trademarks in any other manner that might cause
 # confusion in the marketplace, including but not limited to in advertising,
 # on websites, or on software.
 #
@@ -32,10 +32,9 @@ produced outputs and execution order.
 """
 import copy
 import json
-import warnings
 from collections import Counter, defaultdict
 from itertools import chain
-from typing import Callable, Dict, Iterable, List, Optional, Set, Tuple, Union
+from typing import Callable, Dict, Iterable, List, Set, Tuple, Union
 
 from toposort import CircularDependencyError as ToposortCircleError
 from toposort import toposort
@@ -44,7 +43,6 @@ import kedro
 from kedro.pipeline.node import Node, _to_list
 
 TRANSCODING_SEPARATOR = "@"
-PARAMETER_KEYWORDS = ("params:", "parameters")
 
 
 def _transcode_split(element: str) -> Tuple[str, str]:
@@ -71,25 +69,7 @@ def _transcode_split(element: str) -> Tuple[str, str]:
     return tuple(split_name)  # type: ignore
 
 
-def _transcode_join(parts: Tuple[str, str]) -> str:
-    """Join the name parts using the transcoding separator.
-    If the transcoding part is missing, the resulting name wil not have it as well.
-
-    Raises:
-        ValueError: wrong number of parts have been provided.
-
-    Returns:
-        Node input/output name.
-    """
-    if not parts or len(parts) > 2:
-        raise ValueError("1 or 2 parts are expected: {}".format(parts))
-    if len(parts) == 1 or not parts[1]:
-        return parts[0]
-
-    return TRANSCODING_SEPARATOR.join(parts)
-
-
-def _get_transcode_compatible_name(element: str) -> str:
+def _strip_transcoding(element: str) -> str:
     """Strip out the transcoding separator and anything that follows.
 
     Returns:
@@ -123,13 +103,10 @@ class Pipeline:
     outputs and execution order.
     """
 
-    # pylint: disable=too-many-public-methods
-
     def __init__(
         self,
         nodes: Iterable[Union[Node, "Pipeline"]],
         *,
-        name: str = None,
         tags: Union[str, Iterable[str]] = None
     ):
         """Initialise ``Pipeline`` with a list of ``Node`` instances.
@@ -139,9 +116,6 @@ class Pipeline:
                 provide pipelines among the list of nodes, those pipelines will
                 be expanded and all their nodes will become part of this
                 new pipeline.
-            name: (DEPRECATED, use `tags` method instead) The name of the pipeline.
-                If specified, this name will be used to tag all of the nodes
-                in the pipeline.
             tags: Optional set of tags to be applied to all the pipeline nodes.
 
         Raises:
@@ -191,17 +165,8 @@ class Pipeline:
         _validate_transcoded_inputs_outputs(nodes)
         _tags = set(_to_list(tags))
 
-        if name:
-            warnings.warn(
-                "`name` parameter is deprecated for the `Pipeline`"
-                " constructor, use `Pipeline.tag` method instead",
-                DeprecationWarning,
-            )
-            _tags.add(name)
-
         nodes = [n.tag(_tags) for n in nodes]
 
-        self._name = name
         self._nodes_by_name = {node.name: node for node in nodes}
         _validate_unique_outputs(nodes)
         _validate_unique_confirms(nodes)
@@ -210,13 +175,13 @@ class Pipeline:
         self._nodes_by_input = defaultdict(set)  # type: Dict[str, Set[Node]]
         for node in nodes:
             for input_ in node.inputs:
-                self._nodes_by_input[_get_transcode_compatible_name(input_)].add(node)
+                self._nodes_by_input[_strip_transcoding(input_)].add(node)
 
         # output -> node with output
         self._nodes_by_output = {}  # type: Dict[str, Node]
         for node in nodes:
             for output in node.outputs:
-                self._nodes_by_output[_get_transcode_compatible_name(output)] = node
+                self._nodes_by_output[_strip_transcoding(output)] = node
 
         self._nodes = nodes
         self._topo_sorted_nodes = _topologically_sorted(self.node_dependencies)
@@ -273,12 +238,10 @@ class Pipeline:
         return set.union(set(), *[node.outputs for node in self.nodes])
 
     def _remove_intermediates(self, datasets: Set[str]) -> Set[str]:
-        intermediate = {
-            _get_transcode_compatible_name(i) for i in self.all_inputs()
-        } & {_get_transcode_compatible_name(o) for o in self.all_outputs()}
-        return {
-            d for d in datasets if _get_transcode_compatible_name(d) not in intermediate
+        intermediate = {_strip_transcoding(i) for i in self.all_inputs()} & {
+            _strip_transcoding(o) for o in self.all_outputs()
         }
+        return {d for d in datasets if _strip_transcoding(d) not in intermediate}
 
     def inputs(self) -> Set[str]:
         """The names of free inputs that must be provided at runtime so that
@@ -314,7 +277,7 @@ class Pipeline:
         return self.all_outputs() | self.all_inputs()
 
     def _transcode_compatible_names(self):
-        return {_get_transcode_compatible_name(ds) for ds in self.data_sets()}
+        return {_strip_transcoding(ds) for ds in self.data_sets()}
 
     def describe(self, names_only: bool = True) -> str:
         """Obtain the order of execution and expected free input variables in
@@ -365,34 +328,16 @@ class Pipeline:
 
         str_representation = (
             "#### Pipeline execution order ####\n"
-            "Name: {0}\n"
-            "Inputs: {1}\n\n"
-            "{2}\n\n"
-            "Outputs: {3}\n"
+            "Inputs: {0}\n\n"
+            "{1}\n\n"
+            "Outputs: {2}\n"
             "##################################"
         )
 
         return str_representation.format(
-            self._name,
-            set_to_string(self.inputs()),
-            nodes_as_string,
-            set_to_string(self.outputs()),
+            set_to_string(self.inputs()), nodes_as_string, set_to_string(self.outputs())
         )
 
-    @property
-    def name(self) -> Optional[str]:
-        """(DEPRECATED, use `Pipeline.tag` method instead) Get the pipeline name.
-
-        Returns:
-            The name of the pipeline as provided in the constructor.
-
-        """
-        warnings.warn(
-            "`Pipeline.name` is deprecated, use `Pipeline.tag` method instead.",
-            DeprecationWarning,
-        )
-        return self._name
-
     @property
     def node_dependencies(self) -> Dict[Node, Set[Node]]:
         """All dependencies of nodes where the first Node has a direct dependency on
@@ -407,9 +352,7 @@ class Pipeline:
         }  # type: Dict[Node, Set[Node]]
         for parent in self._nodes:
             for output in parent.outputs:
-                for child in self._nodes_by_input[
-                    _get_transcode_compatible_name(output)
-                ]:
+                for child in self._nodes_by_input[_strip_transcoding(output)]:
                     dependencies[child].add(parent)
 
         return dependencies
@@ -489,14 +432,10 @@ class Pipeline:
 
         relevant_nodes = set()
         for input_ in datasets:
-            if _get_transcode_compatible_name(input_) == input_:
-                relevant_nodes.update(
-                    self._nodes_by_input[_get_transcode_compatible_name(input_)]
-                )
+            if _strip_transcoding(input_) == input_:
+                relevant_nodes.update(self._nodes_by_input[_strip_transcoding(input_)])
             else:
-                for node_ in self._nodes_by_input[
-                    _get_transcode_compatible_name(input_)
-                ]:
+                for node_ in self._nodes_by_input[_strip_transcoding(input_)]:
                     if input_ in node_.inputs:
                         relevant_nodes.add(node_)
         return relevant_nodes
@@ -526,12 +465,10 @@ class Pipeline:
 
         relevant_nodes = set()
         for output in datasets:
-            if _get_transcode_compatible_name(output) in self._nodes_by_output:
-                node_with_output = self._nodes_by_output[
-                    _get_transcode_compatible_name(output)
-                ]
+            if _strip_transcoding(output) in self._nodes_by_output:
+                node_with_output = self._nodes_by_output[_strip_transcoding(output)]
                 if (
-                    _get_transcode_compatible_name(output) == output
+                    _strip_transcoding(output) == output
                     or output in node_with_output.outputs
                 ):
                     relevant_nodes.add(node_with_output)
@@ -597,7 +534,7 @@ class Pipeline:
 
             next_nodes = set(
                 chain.from_iterable(
-                    self._nodes_by_input[_get_transcode_compatible_name(input_)]
+                    self._nodes_by_input[_strip_transcoding(input_)]
                     for input_ in starting
                 )
             )
@@ -661,9 +598,9 @@ class Pipeline:
             starting = inputs
 
             next_nodes = {
-                self._nodes_by_output[_get_transcode_compatible_name(output)]
+                self._nodes_by_output[_strip_transcoding(output)]
                 for output in starting
-                if _get_transcode_compatible_name(output) in self._nodes_by_output
+                if _strip_transcoding(output) in self._nodes_by_output
             }
 
         return Pipeline(result)
@@ -686,7 +623,7 @@ class Pipeline:
         """
 
         res = self.only_nodes(*node_names)
-        res += self.from_inputs(*map(_get_transcode_compatible_name, res.all_outputs()))
+        res += self.from_inputs(*map(_strip_transcoding, res.all_outputs()))
         return res
 
     def to_nodes(self, *node_names: str) -> "Pipeline":
@@ -707,7 +644,7 @@ class Pipeline:
         """
 
         res = self.only_nodes(*node_names)
-        res += self.to_outputs(*map(_get_transcode_compatible_name, res.all_inputs()))
+        res += self.to_outputs(*map(_strip_transcoding, res.all_inputs()))
         return res
 
     def only_nodes_with_tags(self, *tags: str) -> "Pipeline":
@@ -771,86 +708,6 @@ class Pipeline:
 
         return json.dumps(pipeline_versioned)
 
-    def transform(
-        self, datasets: Dict[str, str] = None, prefix: str = None
-    ) -> "Pipeline":
-        """Create a copy of the pipeline and its nodes,
-        with some dataset names modified.
-
-        Args:
-            datasets: A map of the existing dataset name to the new one.
-                Both input and output datasets can be replaced this way.
-            prefix: A prefix to give to all dataset names,
-                except those explicitly named with the `datasets` parameter,
-                and parameter references (`params:` and `parameters`).
-
-        Raises:
-            ValueError: invalid dataset names are given.
-
-        Returns:
-            A new ``Pipeline`` object with the new nodes, modified as requested.
-        """
-        # pylint: disable=protected-access
-        datasets = datasets or {}
-        new_nodes = []
-        used_dataset_names = set()
-
-        def _prefix(name):
-            if any(param in name for param in PARAMETER_KEYWORDS):
-                return name
-            return "{}.{}".format(prefix, name) if prefix else name
-
-        def _map_and_prefix(name):
-            if name in datasets:
-                used_dataset_names.add(name)
-                return datasets[name]
-
-            base_name, transcode_name = _transcode_split(name)
-
-            if base_name in datasets:
-                used_dataset_names.add(base_name)
-                base_name = datasets[base_name]
-                return _transcode_join((base_name, transcode_name))
-
-            return _prefix(name)
-
-        def _process_dataset_names(names: Union[None, str, List[str], Dict[str, str]]):
-            if names is None:
-                return None
-            if isinstance(names, str):
-                return _map_and_prefix(names)  # type: ignore
-            if isinstance(names, list):
-                return [
-                    _map_and_prefix(name) for name in names  # type: ignore
-                ]
-            if isinstance(names, dict):
-                return {
-                    key: _map_and_prefix(value)  # type: ignore
-                    for key, value in names.items()
-                }
-
-            raise ValueError(  # pragma: no cover
-                "Unexpected input {} of type {}".format(names, type(names))
-            )
-
-        for node in self.nodes:
-            new_nodes.append(
-                node._copy(
-                    inputs=_process_dataset_names(node._inputs),
-                    outputs=_process_dataset_names(node._outputs),
-                    name=_prefix(node._name) if node._name else None,
-                )
-            )
-
-        unused_dataset_names = set(datasets) - used_dataset_names
-
-        if unused_dataset_names:
-            raise ValueError(
-                "Failed to map datasets: {}".format(sorted(unused_dataset_names))
-            )
-
-        return Pipeline(new_nodes)
-
 
 def _validate_no_node_list(nodes: Iterable[Union[Node, Pipeline]]):
     if nodes is None:
@@ -874,7 +731,7 @@ def _validate_duplicate_nodes(nodes: List[Node]):
 
 def _validate_unique_outputs(nodes: List[Node]) -> None:
     outputs = chain.from_iterable(node.outputs for node in nodes)
-    outputs = map(_get_transcode_compatible_name, outputs)
+    outputs = map(_strip_transcoding, outputs)
     duplicates = [key for key, value in Counter(outputs).items() if value > 1]
     if duplicates:
         raise OutputNotUniqueError(
@@ -885,7 +742,7 @@ def _validate_unique_outputs(nodes: List[Node]) -> None:
 
 def _validate_unique_confirms(nodes: List[Node]) -> None:
     confirms = chain.from_iterable(node.confirms for node in nodes)
-    confirms = map(_get_transcode_compatible_name, confirms)
+    confirms = map(_strip_transcoding, confirms)
     duplicates = [key for key, value in Counter(confirms).items() if value > 1]
     if duplicates:
         raise ConfirmNotUniqueError(
@@ -907,7 +764,7 @@ def _validate_transcoded_inputs_outputs(nodes: List[Node]) -> None:
 
     invalid = set()
     for dataset_name in all_inputs_outputs:
-        name = _get_transcode_compatible_name(dataset_name)
+        name = _strip_transcoding(dataset_name)
         if name != dataset_name and name in all_inputs_outputs:
             invalid.add(name)
 
